//
// Copyright 2021 Vulcanize, Inc.
//

import { Entity, PrimaryColumn, Column, Index } from 'typeorm';
import { decimalTransformer, bigintTransformer } from '@cerc-io/util';
import { Decimal } from 'decimal.js';

@Entity()
@Index(['blockNumber'])
export class Vault {
  @PrimaryColumn('varchar')
    id!: string;

  @PrimaryColumn('varchar', { length: 66 })
    blockHash!: string;

  @Column('integer')
    blockNumber!: number;

  @Column('varchar')
    deployer!: string;

  @Column('varchar')
    token0!: string;

  @Column('varchar')
    token1!: string;

  @Column('numeric', { transformer: decimalTransformer })
    totalAmount0!: Decimal;

  @Column('numeric', { transformer: decimalTransformer })
    totalAmount1!: Decimal;

  @Column('numeric', { transformer: bigintTransformer })
    createdAt!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    lastSnapshot!: bigint;

  @Column('varchar')
    pool!: string;

  @Column('numeric', { transformer: bigintTransformer })
    state!: bigint;

  @Column('varchar')
    strategyToken!: string;

  @Column('numeric', { transformer: decimalTransformer })
    annualPercentageYield!: Decimal;

  @Column('numeric', { transformer: decimalTransformer })
    annualPercentageDailyYield!: Decimal;

  @Column('numeric', { transformer: decimalTransformer })
    annualPercentageMonthlyYield!: Decimal;

  @Column('numeric', { transformer: decimalTransformer })
    annualPercentageYearlyYield!: Decimal;

  @Column('numeric', { transformer: decimalTransformer })
    lastTotalT0ValuePerLPT!: Decimal;

  @Column('numeric', { transformer: bigintTransformer })
    accruedStrategistFees0!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    accruedStrategistFees1!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    fees0!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    fees1!: bigint;

  @Column('varchar')
    beaconName!: string;

  @Column('numeric', { transformer: bigintTransformer })
    gasUsed!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    gasDeposited!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    totalLPTokensIssued!: bigint;

  @Column('varchar')
    token1Symbol!: string;

  @Column('numeric', { transformer: bigintTransformer })
    decimals!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    feeTier!: bigint;

  @Column('varchar')
    name!: string;

  @Column('varchar')
    symbol!: string;

  @Column('numeric', { transformer: bigintTransformer })
    token0Balance!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    token0Decimals!: bigint;

  @Column('varchar')
    token0Name!: string;

  @Column('varchar')
    token0Symbol!: string;

  @Column('numeric', { transformer: bigintTransformer })
    token1Balance!: bigint;

  @Column('numeric', { transformer: bigintTransformer })
    token1Decimals!: bigint;

  @Column('varchar')
    token1Name!: string;

  @Column('varchar')
    payloadIpfs!: string;

  @Column('varchar')
    vaultManager!: string;

  @Column('numeric', { transformer: decimalTransformer })
    averageFeeArrPerSecond!: Decimal;

  @Column('numeric', { transformer: bigintTransformer })
    totalSnapshots!: bigint;

  @Column('numeric', { transformer: decimalTransformer })
    annualFeeARR!: Decimal;

  @Column('numeric', { transformer: decimalTransformer })
    dailyFeeAPR!: Decimal;

  @Column('numeric', { transformer: decimalTransformer })
    weeklyFeeAPR!: Decimal;

  @Column('numeric', { transformer: decimalTransformer })
    totalValueLockedToken0!: Decimal;

  @Column('numeric', { transformer: decimalTransformer })
    totalValueLockedToken1!: Decimal;

  @Column('boolean', { default: false })
    isPruned!: boolean;

  @Column('boolean', { default: false })
    isRemoved!: boolean;
}
